@@ -1,8 +1,8 @@
 /*****************************************************************************
- * vlc_codec.h: codec related structures
+ * vlc_codec.h: Definition of the decoder and encoder structures
  *****************************************************************************
- * Copyright (C) 1999-2003 VideoLAN
- * $Id: vlc_codec.h 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 1999-2003 the VideoLAN team
+ * $Id$
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -18,10 +18,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
-#ifndef _VLC_CODEC_H
-#define _VLC_CODEC_H 1
+
+#ifndef VLC_CODEC_H
+#define VLC_CODEC_H 1
+
+#include <vlc_block.h>
+#include <vlc_es.h>
 
 /**
  * \file
@@ -38,6 +42,11 @@ typedef struct decoder_owner_sys_t decoder_owner_sys_t;
  * @{
  */
 
+/*
+ * BIG FAT WARNING : the code relies in the first 4 members of filter_t
+ * and decoder_t to be the same, so if you have anything to add, do it
+ * at the end of the structure.
+ */
 struct decoder_t
 {
     VLC_COMMON_MEMBERS
@@ -53,16 +62,25 @@ struct decoder_t
     es_format_t         fmt_out;
 
     /* Some decoders only accept packetized data (ie. not truncated) */
-    vlc_bool_t          b_need_packetized;
+    bool          b_need_packetized;
 
     /* Tell the decoder if it is allowed to drop frames */
-    vlc_bool_t          b_pace_control;
+    bool          b_pace_control;
 
+    /* */
     picture_t *         ( * pf_decode_video )( decoder_t *, block_t ** );
     aout_buffer_t *     ( * pf_decode_audio )( decoder_t *, block_t ** );
     subpicture_t *      ( * pf_decode_sub)   ( decoder_t *, block_t ** );
     block_t *           ( * pf_packetize )   ( decoder_t *, block_t ** );
 
+    /* Closed Caption (CEA 608/708) extraction.
+     * If set, it *may* be called after pf_decode_video/pf_packetize
+     * returned data. It should return CC for the pictures returned by the
+     * last pf_packetize/pf_decode_video call only,
+     * pb_present will be used to known which cc channel are present (but
+     * globaly, not necessary for the current packet */
+    block_t *           ( * pf_get_cc )      ( decoder_t *, bool pb_present[4] );
+
     /*
      * Buffers allocation
      */
@@ -122,11 +140,15 @@ struct encoder_t
     int i_tolerance;             /* Bitrate tolerance */
 
     /* Encoder config */
-    sout_cfg_t *p_cfg;
+    config_chain_t *p_cfg;
 };
 
 /**
  * @}
  */
 
+VLC_EXPORT( input_attachment_t *, decoder_GetInputAttachment, ( decoder_t *, const char *psz_name ) );
+VLC_EXPORT( int, decoder_GetInputAttachments, ( decoder_t *p_dec, input_attachment_t ***ppp_attachment, int *pi_attachment ) );
+VLC_EXPORT( mtime_t, decoder_GetDisplayDate, ( decoder_t *, mtime_t ) );
+
 #endif /* _VLC_CODEC_H */
