@@ -1,8 +1,8 @@
 /*****************************************************************************
- * vlc_es_out.h
+ * vlc_es_out.h: es_out (demuxer output) descriptor, queries and methods
  *****************************************************************************
- * Copyright (C) 1999-2004 VideoLAN
- * $Id: vlc_es_out.h 10429 2005-03-25 09:16:24Z fenrir $
+ * Copyright (C) 1999-2004 the VideoLAN team
+ * $Id: bd654484a972d30c5b23de24c00dead6a9cd90fe $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -18,11 +18,16 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#ifndef _VLC_ES_OUT_H
-#define _VLC_ES_OUT_H 1
+#ifndef VLC_ES_OUT_H
+#define VLC_ES_OUT_H 1
+
+/**
+ * \file
+ * This file defines functions and structures for handling es_out in stream output
+ */
 
 /**
  * \defgroup es out Es Out
@@ -39,31 +44,35 @@ enum es_out_mode_e
 
 enum es_out_query_e
 {
-    /* activate apply of mode */
-    ES_OUT_SET_ACTIVE,  /* arg1= vlc_bool_t                     */
+    /* activate application of mode */
+    ES_OUT_SET_ACTIVE,  /* arg1= bool                     */
     /* see if mode is currently aplied or not */
-    ES_OUT_GET_ACTIVE,  /* arg1= vlc_bool_t*                    */
+    ES_OUT_GET_ACTIVE,  /* arg1= bool*                    */
 
     /* set/get mode */
     ES_OUT_SET_MODE,    /* arg1= int                            */
     ES_OUT_GET_MODE,    /* arg2= int*                           */
 
-    /* set es selected for the es category(audio/video/spu) */
+    /* set ES selected for the es category (audio/video/spu) */
     ES_OUT_SET_ES,      /* arg1= es_out_id_t*                   */
 
-    /* force selection/unselection of the ES (bypass current mode)*/
-    ES_OUT_SET_ES_STATE,/* arg1= es_out_id_t* arg2=vlc_bool_t   */
-    ES_OUT_GET_ES_STATE,/* arg1= es_out_id_t* arg2=vlc_bool_t*  */
+    /* set 'default' tag on ES (copied across from container) */
+    ES_OUT_SET_DEFAULT, /* arg1= es_out_id_t*                   */
+
+    /* force selection/unselection of the ES (bypass current mode) */
+    ES_OUT_SET_ES_STATE,/* arg1= es_out_id_t* arg2=bool   */
+    ES_OUT_GET_ES_STATE,/* arg1= es_out_id_t* arg2=bool*  */
 
     /* */
     ES_OUT_SET_GROUP,   /* arg1= int                            */
     ES_OUT_GET_GROUP,   /* arg1= int*                           */
 
-    /* PCR handling, dts/pts will be automatically computed using thoses PCR
-     * XXX: SET_PCR(_GROUP) is in charge of the pace control. They will wait to slow
-     * down the demuxer to read at the right speed.
-     * XXX: if you want PREROLL just call RESET_PCR and ES_OUT_SET_NEXT_DISPLAY_TIME and send
-     * data to the decoder *without* calling SET_PCR until preroll is finished.
+    /* PCR handling, DTS/PTS will be automatically computed using thoses PCR
+     * XXX: SET_PCR(_GROUP) are in charge of the pace control. They will wait
+     * to slow down the demuxer so that it reads at the right speed.
+     * XXX: if you want PREROLL just call RESET_PCR and
+     * ES_OUT_SET_NEXT_DISPLAY_TIME and send data to the decoder *without*
+     * calling SET_PCR until preroll is finished.
      */
     ES_OUT_SET_PCR,             /* arg1=int64_t i_pcr(microsecond!) (using default group 0)*/
     ES_OUT_SET_GROUP_PCR,       /* arg1= int i_group, arg2=int64_t i_pcr(microsecond!)*/
@@ -80,6 +89,10 @@ enum es_out_query_e
     ES_OUT_SET_NEXT_DISPLAY_TIME,   /* arg1=es_out_id_t* arg2=int64_t i_pts(microsecond) */
     /* Set meta data for group (dynamic) */
     ES_OUT_SET_GROUP_META,  /* arg1=int i_group arg2=vlc_meta_t */
+    /* Set epg for group (dynamic) */
+    ES_OUT_SET_GROUP_EPG,   /* arg1=int i_group arg2=vlc_epg_t */
+    /* */
+    ES_OUT_DEL_GROUP        /* arg1=int i_group */
 };
 
 struct es_out_t
@@ -88,6 +101,7 @@ struct es_out_t
     int          (*pf_send)   ( es_out_t *, es_out_id_t *, block_t * );
     void         (*pf_del)    ( es_out_t *, es_out_id_t * );
     int          (*pf_control)( es_out_t *, int i_query, va_list );
+    bool      b_sout;
 
     es_out_sys_t    *p_sys;
 };
@@ -96,10 +110,12 @@ static inline es_out_id_t * es_out_Add( es_out_t *out, es_format_t *fmt )
 {
     return out->pf_add( out, fmt );
 }
+
 static inline void es_out_Del( es_out_t *out, es_out_id_t *id )
 {
     out->pf_del( out, id );
 }
+
 static inline int es_out_Send( es_out_t *out, es_out_id_t *id,
                                block_t *p_block )
 {
@@ -110,6 +126,7 @@ static inline int es_out_vaControl( es_out_t *out, int i_query, va_list args )
 {
     return out->pf_control( out, i_query, args );
 }
+
 static inline int es_out_Control( es_out_t *out, int i_query, ... )
 {
     va_list args;
