@@ -2,7 +2,7 @@
  * filter.c : DirectShow access module for vlc
  *****************************************************************************
  * Copyright (C) 2002 the VideoLAN team
- * $Id: filter.cpp 13905 2006-01-12 23:10:04Z dionoea $
+ * $Id: 37c34c7721de76ac899a52d9145033fd5289108e $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -24,13 +24,13 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 
-#include <vlc/vlc.h>
-#include <vlc/input.h>
-#include <vlc/vout.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_vout.h>
 
 #ifndef _MSC_VER
     /* Work-around a bug in w32api-2.5 */
@@ -144,6 +144,12 @@ const GUID MEDIASUBTYPE_MPEG2_PROGRAM = {0xe06d8022, 0xdb46, 0x11cf, {0xb4, 0xd1
 const GUID MEDIASUBTYPE_MPEG2_TRANSPORT = {0xe06d8023, 0xdb46, 0x11cf, {0xb4, 0xd1, 0x00, 0x80, 0x5f, 0x6c, 0xbb, 0xea}};
 const GUID FORMAT_MPEG2Video = {0xe06d80e3, 0xdb46, 0x11cf, {0xb4, 0xd1, 0x00, 0x80, 0x5f, 0x6c, 0xbb, 0xea}};
 
+/* MJPG format */
+const GUID MEDIASUBTYPE_MJPG = {0x47504A4D, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
+
+/* DivX formats */
+const GUID MEDIASUBTYPE_DIVX = {0x58564944, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}};
+
 /* Analog Video */
 const GUID FORMAT_AnalogVideo = {0x482dde0, 0x7817, 0x11cf, {0x8a, 0x3, 0x0, 0xaa, 0x0, 0x6e, 0xcb, 0x65}};
 
@@ -275,6 +281,10 @@ int GetFourCCFromMediaType( const AM_MEDIA_TYPE &media_type )
             else if( media_type.subtype == MEDIASUBTYPE_MPEG2_VIDEO )
                i_fourcc = VLC_FOURCC( 'm', 'p', '2', 'v' );
 
+        /* DivX video */
+            else if( media_type.subtype == MEDIASUBTYPE_DIVX )
+               i_fourcc = VLC_FOURCC( 'D', 'I', 'V', 'X' );
+
             /* DV formats */
             else if( media_type.subtype == MEDIASUBTYPE_dvsl )
                i_fourcc = VLC_FOURCC( 'd', 'v', 's', 'l' );
@@ -282,6 +292,11 @@ int GetFourCCFromMediaType( const AM_MEDIA_TYPE &media_type )
                i_fourcc = VLC_FOURCC( 'd', 'v', 's', 'd' );
             else if( media_type.subtype == MEDIASUBTYPE_dvhd )
                i_fourcc = VLC_FOURCC( 'd', 'v', 'h', 'd' );
+
+            /* MJPEG format */
+            else if( media_type.subtype == MEDIASUBTYPE_MJPG )
+                i_fourcc = VLC_FOURCC( 'M', 'J', 'P', 'G' );
+
         }
     }
     else if( media_type.majortype == MEDIATYPE_Audio )
@@ -431,7 +446,7 @@ STDMETHODIMP CapturePin::Connect( IPin * pReceivePin,
     }
 
     if( !pmt ) return S_OK;
-                
+ 
     if( GUID_NULL != pmt->majortype &&
         media_types[0].majortype != pmt->majortype )
     {
@@ -589,7 +604,7 @@ STDMETHODIMP CapturePin::QueryAccept( const AM_MEDIA_TYPE *pmt )
 
     int i_fourcc = GetFourCCFromMediaType(*pmt);
     if( !i_fourcc )
-    {   
+    {
         msg_Dbg( p_input, "CapturePin::QueryAccept "
                  "[media type not supported]" );
         return S_FALSE;
@@ -606,10 +621,11 @@ STDMETHODIMP CapturePin::QueryAccept( const AM_MEDIA_TYPE *pmt )
         }
 
         msg_Dbg( p_input, "CapturePin::QueryAccept [OK] "
-                 "(width=%ld, height=%ld, chroma=%4.4s)",
+                 "(width=%ld, height=%ld, chroma=%4.4s, fps=%f)",
                  ((VIDEOINFOHEADER *)pmt->pbFormat)->bmiHeader.biWidth,
                  ((VIDEOINFOHEADER *)pmt->pbFormat)->bmiHeader.biHeight,
-                 (char *)&i_fourcc );
+                 (char *)&i_fourcc,
+         10000000.0f/((float)((VIDEOINFOHEADER *)pmt->pbFormat)->AvgTimePerFrame) );
     }
     else if( pmt->majortype == MEDIATYPE_Audio )
     {
@@ -619,7 +635,7 @@ STDMETHODIMP CapturePin::QueryAccept( const AM_MEDIA_TYPE *pmt )
                  ((WAVEFORMATEX *)pmt->pbFormat)->nSamplesPerSec,
                  ((WAVEFORMATEX *)pmt->pbFormat)->wBitsPerSample,
                  (char *)&i_fourcc );
-    }   
+    }
     else
     {
         msg_Dbg( p_input, "CapturePin::QueryAccept [OK] (stream format=%4.4s)",
@@ -780,7 +796,7 @@ CaptureFilter::CaptureFilter( vlc_object_t *_p_input, access_sys_t *p_sys,
                               AM_MEDIA_TYPE *mt, size_t mt_count )
   : p_input( _p_input ),
     p_pin( new CapturePin( _p_input, p_sys, this, mt, mt_count ) ),
-    state( State_Stopped ), i_ref( 1 ) 
+    state( State_Stopped ), i_ref( 1 )
 {
 }
 
@@ -1122,12 +1138,12 @@ CaptureEnumMediaTypes::CaptureEnumMediaTypes( vlc_object_t *_p_input,
     /* Are we creating a new enumerator */
     if( pEnumMediaTypes == NULL )
     {
-        CopyMediaType(&cx_media_type, &p_pin->cx_media_type); 
+        CopyMediaType(&cx_media_type, &p_pin->cx_media_type);
         i_position = 0;
     }
     else
     {
-        CopyMediaType(&cx_media_type, &pEnumMediaTypes->cx_media_type); 
+        CopyMediaType(&cx_media_type, &pEnumMediaTypes->cx_media_type);
         i_position = pEnumMediaTypes->i_position;
     }
 }
@@ -1192,7 +1208,7 @@ STDMETHODIMP CaptureEnumMediaTypes::Next( ULONG cMediaTypes,
     ULONG offset = 0;
     ULONG max = p_pin->media_type_count;
 
-    if( ! ppMediaTypes ) 
+    if( ! ppMediaTypes )
         return E_POINTER;
 
     if( (! pcFetched)  && (cMediaTypes > 1) )
@@ -1211,7 +1227,7 @@ STDMETHODIMP CaptureEnumMediaTypes::Next( ULONG cMediaTypes,
                 (AM_MEDIA_TYPE *)CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
             if( CopyMediaType(ppMediaTypes[copied], &cx_media_type) != S_OK )
                 return E_OUTOFMEMORY;
-            ++i_position; 
+            ++i_position;
             ++copied;
         }
     }
@@ -1225,7 +1241,7 @@ STDMETHODIMP CaptureEnumMediaTypes::Next( ULONG cMediaTypes,
             return E_OUTOFMEMORY;
 
         ++copied;
-        ++i_position; 
+        ++i_position;
     }
 
     if( pcFetched )  *pcFetched = copied;
@@ -1253,7 +1269,7 @@ STDMETHODIMP CaptureEnumMediaTypes::Reset()
 #endif
 
     FreeMediaType(cx_media_type);
-    CopyMediaType(&cx_media_type, &p_pin->cx_media_type); 
+    CopyMediaType(&cx_media_type, &p_pin->cx_media_type);
     i_position = 0;
     return S_OK;
 };
