@@ -2,7 +2,7 @@
  * mms.h: MMS access plug-in
  *****************************************************************************
  * Copyright (C) 2001, 2002 the VideoLAN team
- * $Id: mms.h 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -18,21 +18,24 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+#ifndef _MMS_H_
+#define _MMS_H_ 1
+
 #define MMS_PROTO_AUTO  0
 #define MMS_PROTO_TCP   1
 #define MMS_PROTO_UDP   2
 #define MMS_PROTO_HTTP  3
 
-
 /* mmst and mmsu */
-int  E_( MMSTUOpen )  ( access_t * );
-void E_( MMSTUClose ) ( access_t * );
+int   MMSTUOpen   ( access_t * );
+void  MMSTUClose  ( access_t * );
 
 /* mmsh */
-int  E_( MMSHOpen )  ( access_t * );
-void E_( MMSHClose ) ( access_t * );
+int   MMSHOpen   ( access_t * );
+void  MMSHClose  ( access_t * );
+
+#endif
 
-#define FREE( p ) if( p ) { free( p ); (p) = NULL; }
