@@ -1,10 +1,10 @@
 /*****************************************************************************
  * demux.c: demux functions for dvdplay.
  *****************************************************************************
- * Copyright (C) 1998-2001 VideoLAN
- * $Id: demux.c 7936 2004-06-07 18:32:12Z fenrir $
+ * Copyright (C) 1998-2001 the VideoLAN team
+ * $Id: 5f534c191ca4724a0ad98f0190ecbe295569eb19 $
  *
- * Author: Stphane Borel <stef@via.ecp.fr>
+ * Author: Stéphane Borel <stef@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,21 +18,22 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
-#include <vlc/vlc.h>
-#include <vlc/input.h>
-#include <vlc/intf.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
-#include "../../demux/mpeg/system.h"
+#include <stddef.h>
+#include <vlc_common.h>
+#include <vlc_input.h>
+#include <vlc_access.h>
+#include <vlc_interface.h>
 
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
@@ -41,7 +42,6 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <string.h>
 #include <errno.h>
 
 #ifdef STRNCASECMP_IN_STRINGS_H
@@ -72,16 +72,16 @@ struct demux_sys_t
 };
 
 /*****************************************************************************
- * InitVCD: initializes structures
+ * VCDInit: initializes structures
  *****************************************************************************/
-int E_(InitVCD) ( vlc_object_t *p_this )
+int VCDInit ( vlc_object_t *p_this )
 {
     input_thread_t *p_input = (input_thread_t *)p_this;
-    vcd_data_t *    p_vcd = (vcd_data_t *)p_input->p_access_data;
+    vcd_data_t *    p_vcd = (vcd_data_t *)p_input->p_sys;
     demux_sys_t *   p_demux;
 
-    printf("++++ InitVCD CALLED\n");
-    
+    printf("++++ VCDInit CALLED\n");
+ 
 
     if( p_input->stream.i_method != INPUT_METHOD_VCD )
     {
@@ -115,9 +115,9 @@ int E_(InitVCD) ( vlc_object_t *p_this )
 }
 
 /*****************************************************************************
- * EndVCD: frees unused data
+ * VCDEnd: frees unused data
  *****************************************************************************/
-void E_(EndVCD) ( vlc_object_t *p_this )
+void VCDEnd ( vlc_object_t *p_this )
 {
     input_thread_t *p_input = (input_thread_t *)p_this;
     vcd_data_t *    p_vcd = p_input->p_demux_data->p_vcd;
@@ -129,7 +129,7 @@ void E_(EndVCD) ( vlc_object_t *p_this )
         intf_StopThread( p_intf );
         vlc_object_detach( p_intf );
         vlc_object_release( p_intf );
-        intf_Destroy( p_intf );
+        vlc_object_release( p_intf );
     }
 
     p_vcd->p_intf = NULL;
