@@ -1,10 +1,10 @@
 /*****************************************************************************
  * intf.h: send info to intf.
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: intf.h 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2001 the VideoLAN team
+ * $Id$
  *
- * Author: Stphane Borel <stef@via.ecp.fr>
+ * Author: Stéphane Borel <stef@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+#include <vlc_common.h>
+#include <vlc_input.h>
+#include "vcdplayer.h"
+
 /*****************************************************************************
  * intf_sys_t: description and status of interface
  *****************************************************************************/
@@ -29,17 +33,17 @@ struct intf_sys_t
   input_thread_t *p_input;
   vcdplayer_t    *p_vcdplayer;
 
-  vlc_bool_t      b_still;           /* True if we are in a still frame */
-  vlc_bool_t      b_infinite_still;  /* True if still wait time is infinite */
+  bool      b_still;           /* True if we are in a still frame */
+  bool      b_infinite_still;  /* True if still wait time is infinite */
   mtime_t         m_still_time;      /* Time in microseconds remaining
                                          to wait in still frame.
-				     */
+                     */
 #if FINISHED
   vcdplay_ctrl_t      control;
 #else
   int                 control;
 #endif
-  vlc_bool_t          b_click, b_move, b_key_pressed;
+  bool          b_click, b_move, b_key_pressed;
 };
 
 int vcdIntfStillTime( struct intf_thread_t * p_intf, uint8_t wait_time);
