@@ -125,7 +125,11 @@ int InitAudioDec( decoder_t *p_dec, AVCodecContext *p_context,
 
     p_sys->p_context->block_align = p_dec->fmt_in.audio.i_blockalign;
     p_sys->p_context->bit_rate = p_dec->fmt_in.i_bitrate;
+#if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
     p_sys->p_context->bits_per_sample = p_dec->fmt_in.audio.i_bitspersample;
+#else
+    p_sys->p_context->bits_per_coded_sample = p_dec->fmt_in.audio.i_bitspersample;
+#endif
 
     if( p_dec->fmt_in.i_extra > 0 )
     {
