@@ -2,7 +2,7 @@
  * SSA/ASS subtitle decoder using libass.
  *****************************************************************************
  * Copyright (C) 2008 the VideoLAN team
- * $Id: a16609abe6737bc74473f34c1de337f7e871119e $
+ * $Id: 58da2e7f7f4ca1c95901f89f6027fe99725b6537 $
  *
  * Authors: Laurent Aimar <fenrir@videolan.org>
  *
@@ -43,6 +43,10 @@
 
 #include <ass/ass.h>
 
+#if defined(WIN32)
+#   include <vlc_charset.h>
+#endif
+
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -696,7 +700,31 @@ static ass_handle_t *AssHandleYield( decoder_t *p_dec )
     }
     free( pp_attachments );
 
-    char *psz_font_dir = config_GetCacheDir();
+    char *psz_font_dir = NULL;
+
+#if defined(WIN32)
+    const UINT uPath = GetSystemWindowsDirectoryW( NULL, 0 );
+    if( uPath > 0 )
+    {
+        wchar_t *psw_path = calloc( uPath + 1, sizeof(wchar_t) );
+        if( psw_path )
+        {
+            if( GetSystemWindowsDirectoryW( psw_path, uPath + 1 ) > 0 )
+            {
+                char *psz_tmp = FromWide( psw_path );
+                if( psz_tmp &&
+                    asprintf( &psz_font_dir, "%s\\Fonts", psz_tmp ) < 0 )
+                    psz_font_dir = NULL;
+                free( psz_tmp );
+            }
+            free( psw_path );
+        }
+    }
+#endif
+
+    if( !psz_font_dir )
+        psz_font_dir = config_GetCacheDir();
+
     if( !psz_font_dir )
         goto error;
     ass_set_fonts_dir( p_library, psz_font_dir );
