@@ -2,7 +2,7 @@
  * libmpeg2.c: mpeg2 video decoder module making use of libmpeg2.
  *****************************************************************************
  * Copyright (C) 1999-2001 the VideoLAN team
- * $Id: 13ec4c512a79b7a56c3e5c6e9c826636b7f715bd $
+ * $Id: ec6d3321cbe16e68808ffa4f239dba9cb25b8089 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -33,17 +33,13 @@
 #include <vlc_plugin.h>
 #include <vlc_vout.h>
 #include <vlc_codec.h>
+#include <vlc_block_helper.h>
+#include "../codec/cc.h"
 
 #include <mpeg2.h>
 
 #include <vlc_codec_synchro.h>
 
-/* Aspect ratio (ISO/IEC 13818-2 section 6.3.3, table 6-3) */
-#define AR_SQUARE_PICTURE       1                           /* square pixels */
-#define AR_4_3_PICTURE          2                        /* 4:3 picture (TV) */
-#define AR_16_9_PICTURE         3              /* 16:9 picture (wide screen) */
-#define AR_221_1_PICTURE        4                  /* 2.21:1 picture (movie) */
-
 /*****************************************************************************
  * decoder_sys_t : libmpeg2 decoder descriptor
  *****************************************************************************/
@@ -77,11 +73,16 @@ struct decoder_sys_t
      * Output properties
      */
     decoder_synchro_t *p_synchro;
-    int            i_aspect;
-    int            i_sar_num;
-    int            i_sar_den;
-    mtime_t        i_last_frame_pts;
+    int             i_aspect;
+    int             i_sar_num;
+    int             i_sar_den;
+    mtime_t         i_last_frame_pts;
 
+    /* Closed captioning support */
+    uint32_t        i_cc_flags;
+    mtime_t         i_cc_pts;
+    mtime_t         i_cc_dts;
+    cc_data_t       cc;
 };
 
 /*****************************************************************************
@@ -91,6 +92,7 @@ static int  OpenDecoder( vlc_object_t * );
 static void CloseDecoder( vlc_object_t * );
 
 static picture_t *DecodeBlock( decoder_t *, block_t ** );
+static block_t   *GetCc( decoder_t *p_dec, bool pb_present[4] );
 
 static picture_t *GetNewPicture( decoder_t *, uint8_t ** );
 static void GetAR( decoder_t *p_dec );
@@ -148,6 +150,14 @@ static int OpenDecoder( vlc_object_t *p_this )
     p_sys->b_skip     = 0;
     p_sys->b_preroll = false;
 
+    p_sys->i_cc_pts = 0;
+    p_sys->i_cc_dts = 0;
+    p_sys->i_cc_flags = 0;
+#if MPEG2_RELEASE >= MPEG2_VERSION (0, 5, 0)
+    p_dec->pf_get_cc = GetCc;
+    cc_Init( &p_sys->cc );
+#endif
+
 #if defined( __i386__ ) || defined( __x86_64__ )
     if( vlc_CPU() & CPU_CAPABILITY_MMX )
     {
@@ -222,6 +232,9 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
                 block_Release( p_block );
                 return NULL;
             }
+            if( p_block->i_flags & (BLOCK_FLAG_DISCONTINUITY
+                                      | BLOCK_FLAG_CORRUPTED))
+                cc_Flush( &p_sys->cc );
 
             if( (p_block->i_flags & (BLOCK_FLAG_DISCONTINUITY
                                       | BLOCK_FLAG_CORRUPTED)) &&
@@ -241,7 +254,10 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
                     uint8_t *buf[3];
                     buf[0] = buf[1] = buf[2] = NULL;
                     if( (p_pic = GetNewPicture( p_dec, buf )) == NULL )
+                    {
+                        p_block->i_buffer = 0;
                         break;
+                    }
                     mpeg2_set_buf( p_sys->p_mpeg2dec, buf, p_pic );
                     mpeg2_stride( p_sys->p_mpeg2dec, p_pic->p[Y_PLANE].i_pitch );
                 }
@@ -439,7 +455,21 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
                 }
 
                 mpeg2_set_buf( p_sys->p_mpeg2dec, buf, p_pic );
-        mpeg2_stride( p_sys->p_mpeg2dec, p_pic->p[Y_PLANE].i_pitch );
+                mpeg2_stride( p_sys->p_mpeg2dec, p_pic->p[Y_PLANE].i_pitch );
+            }
+            if( p_sys->p_info->user_data_len > 2 )
+            {
+                p_sys->i_cc_pts = i_pts;
+                p_sys->i_cc_dts = i_dts;
+                if( (p_sys->p_info->current_picture->flags
+                             & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_P )
+                    p_sys->i_cc_flags = BLOCK_FLAG_TYPE_P;
+                else if( (p_sys->p_info->current_picture->flags
+                             & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_B )
+                    p_sys->i_cc_flags = BLOCK_FLAG_TYPE_B;
+                else p_sys->i_cc_flags = BLOCK_FLAG_TYPE_I;
+
+                cc_Extract( &p_sys->cc, &p_sys->p_info->user_data[0], p_sys->p_info->user_data_len );
             }
         }
         break;
@@ -488,7 +518,6 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
 
                 return p_pic;
             }
-
             break;
 
         case STATE_INVALID:
@@ -505,6 +534,7 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
             }
             mpeg2_skip( p_sys->p_mpeg2dec, 1 );
             p_sys->b_skip = 1;
+            cc_Flush( &p_sys->cc );
 
             if( p_sys->p_info->current_fbuf &&
                 p_sys->p_info->current_fbuf->id )
@@ -622,6 +652,33 @@ static picture_t *GetNewPicture( decoder_t *p_dec, uint8_t **pp_buf )
     return p_pic;
 }
 
+/*****************************************************************************
+ * GetCc: Retrieves the Closed Captions for the CC decoder.
+ *****************************************************************************/
+static block_t *GetCc( decoder_t *p_dec, bool pb_present[4] )
+{
+    decoder_sys_t   *p_sys = p_dec->p_sys;
+    block_t         *p_cc = NULL;
+    int i;
+
+    for( i = 0; i < 4; i++ )
+        pb_present[i] = p_sys->cc.pb_present[i];
+
+    if( p_sys->cc.i_data <= 0 )
+        return NULL;
+
+    p_cc = block_New( p_dec, p_sys->cc.i_data);
+    if( p_cc )
+    {
+        memcpy( p_cc->p_buffer, p_sys->cc.p_data, p_sys->cc.i_data );
+        p_cc->i_dts =
+        p_cc->i_pts = p_sys->cc.b_reorder ? p_sys->i_cc_pts : p_sys->i_cc_dts;
+        p_cc->i_flags = ( p_sys->cc.b_reorder  ? p_sys->i_cc_flags : BLOCK_FLAG_TYPE_P ) & ( BLOCK_FLAG_TYPE_I|BLOCK_FLAG_TYPE_P|BLOCK_FLAG_TYPE_B);
+    }
+    cc_Flush( &p_sys->cc );
+    return p_cc;
+}
+
 /*****************************************************************************
  * GetAR: Get aspect ratio
  *****************************************************************************/
@@ -633,31 +690,6 @@ static void GetAR( decoder_t *p_dec )
     if( p_dec->fmt_in.video.i_aspect )
     {
         p_sys->i_aspect = p_dec->fmt_in.video.i_aspect;
-        if( p_sys->i_aspect <= AR_221_1_PICTURE )
-        switch( p_sys->i_aspect )
-        {
-        case AR_4_3_PICTURE:
-            p_sys->i_aspect = VOUT_ASPECT_FACTOR * 4 / 3;
-            p_sys->i_sar_num = p_sys->p_info->sequence->picture_height * 4;
-            p_sys->i_sar_den = p_sys->p_info->sequence->picture_width * 3;
-            break;
-        case AR_16_9_PICTURE:
-            p_sys->i_aspect = VOUT_ASPECT_FACTOR * 16 / 9;
-            p_sys->i_sar_num = p_sys->p_info->sequence->picture_height * 16;
-            p_sys->i_sar_den = p_sys->p_info->sequence->picture_width * 9;
-            break;
-        case AR_221_1_PICTURE:
-            p_sys->i_aspect = VOUT_ASPECT_FACTOR * 221 / 100;
-            p_sys->i_sar_num = p_sys->p_info->sequence->picture_height * 221;
-            p_sys->i_sar_den = p_sys->p_info->sequence->picture_width * 100;
-            break;
-        case AR_SQUARE_PICTURE:
-            p_sys->i_aspect = VOUT_ASPECT_FACTOR *
-                           p_sys->p_info->sequence->picture_width /
-                           p_sys->p_info->sequence->picture_height;
-            p_sys->i_sar_num = p_sys->i_sar_den = 1;
-            break;
-        }
     }
     else
     {
