@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2007 Vincent Penne
  * Some code converted from ProjectX java dvb decoder (c) 2001-2005 by dvb.matt
- * $Id$
+ * $Id: ebcd4467376d866111b52c02675d11e74dba1210 $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -93,15 +93,15 @@ vlc_module_end();
 struct decoder_sys_t
 {
   int         i_align;
-  bool  b_is_subtitle[9];
+  bool        b_is_subtitle[9];
   char        ppsz_lines[32][128];
   char        psz_prev_text[512];
   mtime_t     prev_pts;
   int         i_page[9];
-  bool  b_erase[9];
+  bool        b_erase[9];
   const uint16_t *  pi_active_national_set[9];
   int         i_wanted_page, i_wanted_magazine;
-  bool  b_ignore_sub_flag;
+  bool        b_ignore_sub_flag;
 };
 
 /****************************************************************************
@@ -131,7 +131,7 @@ static const uint16_t ppi_national_subsets[][20] =
     0x00e9, 0x00e4, 0x00f6, 0x00e5, 0x00fc }, /* swedish,finnish,hungarian ,010 */
 
   { 0x0023, 0x016f, 0x010d, 0x0165, 0x017e, 0x00fd, 0x00ed, 0x0159,
-    0x00e9, 0x00e1, 0x0115, 0x00fa, 0x0161 }, /* czech,slovak  ,011 */
+    0x00e9, 0x00e1, 0x011b, 0x00fa, 0x0161 }, /* czech,slovak  ,011 */
 
   { 0x0023, 0x0024, 0x00a7, 0x00c4, 0x00d6, 0x00dc, 0x005e, 0x005f,
     0x00b0, 0x00e4, 0x00f6, 0x00fc, 0x00df }, /* german ,100 */
@@ -193,9 +193,10 @@ static int Open( vlc_object_t *p_this )
         p_sys->pi_active_national_set[i] = ppi_national_subsets[1];
 
     var_Create( p_dec, "telx-override-page",
-                VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+                VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     var_Get( p_dec, "telx-override-page", &val );
-    if( val.i_int == -1 && p_dec->fmt_in.subs.dvb.i_id != -1 )
+    if( val.i_int == -1 && p_dec->fmt_in.subs.dvb.i_id != -1 
+          && p_dec->fmt_in.subs.dvb.i_id != (1<<16) ) /* ignore if TS demux wants page 100 (unlikely to be sub) */
     {
         p_sys->i_wanted_magazine = p_dec->fmt_in.subs.dvb.i_id >> 16;
         if( p_sys->i_wanted_magazine == 0 )
@@ -231,7 +232,7 @@ static int Open( vlc_object_t *p_this )
     var_Get( p_dec, "telx-ignore-subtitle-flag", &val );
     p_sys->b_ignore_sub_flag = val.b_bool;
 
-    msg_Dbg( p_dec, "starting telx on magazine %d page %x flag %d",
+    msg_Dbg( p_dec, "starting telx on magazine %d page %02x flag %d",
              p_sys->i_wanted_magazine, p_sys->i_wanted_page,
              p_sys->b_ignore_sub_flag );
 
