@@ -2,7 +2,7 @@
  * zvbi.c : VBI and Teletext PES demux and decoder using libzvbi
  *****************************************************************************
  * Copyright (C) 2007, M2X
- * $Id$
+ * $Id: 3025d2047ecb00a1b0a557e32ee1f819a4336814 $
  *
  * Authors: Derk-Jan Hartman <djhartman at m2x dot nl>
  *          Jean-Paul Saman <jpsaman at m2x dot nl>
@@ -102,6 +102,27 @@ vlc_module_end();
 
 // #define ZVBI_DEBUG
 
+//Guessing table for missing "default region triplet"
+static const int pi_default_triplet[] = {
+ 0, 0,           // slo cze
+ 8,              // pol
+ 24,24,24,24,    //ssc scr slv rum
+ 32,32,32,32,32, //est lit rus bul ukr
+ 48,48,          //gre ell
+ 64,             //ara
+ 88,             //heb
+ 16 };           //default
+static const char *const ppsz_default_triplet[] = {
+ "slo", "cze",
+ "pol",
+ "ssc", "scr", "slv", "rum",
+ "est", "lit", "rus", "bul", "ukr",
+ "gre", "ell",
+ "ara",
+ "heb",
+ NULL
+};
+
 typedef enum {
     DATA_UNIT_EBU_TELETEXT_NON_SUBTITLE     = 0x02,
     DATA_UNIT_EBU_TELETEXT_SUBTITLE         = 0x03,
@@ -187,6 +208,19 @@ static int Open( vlc_object_t *p_this )
         return VLC_ENOMEM;
     }
 
+    /* Some broadcasters in countries with level 1 and level 1.5 still not send a G0 to do 
+     * matches against table 32 of ETSI 300 706. We try to do some best effort guessing
+     * This is not perfect, but might handle some cases where we know the vbi language 
+     * is known. It would be better if people started sending G0 */
+    for( int i = 0; ppsz_default_triplet[i] != NULL; i++ )
+    {
+        if( p_dec->fmt_in.psz_language && !strcasecmp( p_dec->fmt_in.psz_language, ppsz_default_triplet[i] ) )
+        {
+            vbi_teletext_set_default_region( p_sys->p_vbi_dec, pi_default_triplet[i]);
+            msg_Dbg( p_dec, "overwriting default zvbi region: %d", pi_default_triplet[i] );
+        }
+    }
+
     vbi_event_handler_register( p_sys->p_vbi_dec, VBI_EVENT_TTX_PAGE |
                                 VBI_EVENT_CAPTION | VBI_EVENT_NETWORK |
                                 VBI_EVENT_ASPECT | VBI_EVENT_PROG_INFO,
@@ -197,6 +231,16 @@ static int Open( vlc_object_t *p_this )
     var_AddCallback( p_dec, "vbi-page",
                      RequestPage, p_sys );
 
+    /* Check if the Teletext track has a known "initial page". */
+    if( p_sys->i_wanted_page == 100 && p_dec->fmt_in.subs.dvb.i_id != -1 )
+    {
+        int i_wanted_magazine = p_dec->fmt_in.subs.dvb.i_id >> 16;
+        if( i_wanted_magazine == 0 )
+            i_wanted_magazine = 8;
+        p_sys->i_wanted_page = vbi_bcd2dec(p_dec->fmt_in.subs.dvb.i_id & 0xff);
+        p_sys->i_wanted_page += 100*i_wanted_magazine;
+    }
+
     p_sys->b_opaque = var_CreateGetBool( p_dec, "vbi-opaque" );
     var_AddCallback( p_dec, "vbi-opaque", Opaque, p_sys );
 
@@ -491,26 +535,27 @@ static int OpaquePage( picture_t *p_src, const vbi_page p_page,
         for( x = 0; x < fmt.i_width; x++ )
         {
             const vbi_opacity opacity = p_page.text[ y/10 * p_page.columns + x/12 ].opacity;
+            const int background = p_page.text[ y/10 * p_page.columns + x/12 ].background;
             uint32_t *p_pixel = (uint32_t*)&p_src->p->p_pixels[y * p_src->p->i_pitch + 4*x];
+
             switch( opacity )
             {
             /* Show video instead of this character */
             case VBI_TRANSPARENT_SPACE:
                 *p_pixel = 0;
                 break;
+            /* Display foreground and background color */
             /* To make the boxed text "closed captioning" transparent
              * change true to false.
              */
             case VBI_OPAQUE:
+            /* alpha blend video into background color */
+            case VBI_SEMI_TRANSPARENT:
                 if( b_opaque )
                     break;
-            /* Full text transparency. only foreground color is show */
-            case VBI_TRANSPARENT_FULL:
-                *p_pixel = 0;
-                break;
             /* Transparency for boxed text */
-            case VBI_SEMI_TRANSPARENT:
-                if( (*p_pixel) == 0xff000000 && 0 ) /* Disabled until someone implement borders */
+            case VBI_TRANSPARENT_FULL:
+                if( (*p_pixel) == (0xff000000 | p_page.color_map[background] ) )
                     *p_pixel = 0;
                 break;
             }
