@@ -2,7 +2,7 @@
  * input_manager.hpp : Manage an input and interact with its GUI elements
  ****************************************************************************
  * Copyright (C) 2006-2008 the VideoLAN team
- * $Id$
+ * $Id: 0d0798d4c9075ba0e138278a20ade3e0942e8081 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -44,6 +44,7 @@ static int const ItemRateChanged_Type    = QEvent::User + IMEventType + 5;
 static int const VolumeChanged_Type      = QEvent::User + IMEventType + 6;
 static int const ItemSpuChanged_Type     = QEvent::User + IMEventType + 7;
 static int const ItemTeletextChanged_Type= QEvent::User + IMEventType + 8;
+static int const InterfaceVoutUpdate_Type= QEvent::User + IMEventType + 9;
 
 static int const FullscreenControlToggle_Type = QEvent::User + IMEventType + 10;
 static int const FullscreenControlShow_Type = QEvent::User + IMEventType + 11;
@@ -70,7 +71,7 @@ public:
     void delInput();
     bool hasInput() { return p_input && !p_input->b_dead && vlc_object_alive (p_input); }
     bool hasAudio();
-    bool hasVideo();
+    bool hasVideo() { return hasInput() && b_video; }
 
     QString getName() { return old_name; }
 
@@ -83,6 +84,7 @@ private:
     QString         artUrl;
     int             i_rate;
     bool            b_transparentTelextext;
+    bool            b_video;
 
     void customEvent( QEvent * );
     void addCallbacks();
@@ -95,6 +97,7 @@ private:
     void UpdateSPU();
     void UpdateTeletext();
     void UpdateArt();
+    void UpdateVout();
 
 public slots:
     void setInput( input_thread_t * ); ///< Our controlled input changed
@@ -121,7 +124,7 @@ signals:
     void navigationChanged( int );
     /// Play/pause status
     void statusChanged( int );
-    void artChanged( QString );
+    void artChanged( input_item_t* );
     /// Teletext
     void teletextEnabled( bool );
     void toggleTelexButtons();
@@ -129,6 +132,8 @@ signals:
     void setNewTelexPage( int );
     /// Advanced buttons
     void advControlsSetIcon();
+    /// Vout
+    void voutChanged( bool );
 };
 
 class MainInputManager : public QObject
