@@ -2,7 +2,7 @@
  * main_interface.cpp : Main interface
  ****************************************************************************
  * Copyright (C) 2006-2008 the VideoLAN team
- * $Id: ab1a1e8cbfbdc6d18dc32c02b78de3f7b7588a7a $
+ * $Id: 52cdee55deb155b631338aa93e4106fead08a3c3 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -957,7 +957,7 @@ void MainInterface::toggleUpdateSystrayMenu()
     }
     else
     {
-        /* Visible */
+        /* Visible (possibly under other windows) */
 #ifdef WIN32
         /* check if any visible window is above vlc in the z-order,
          * but ignore the ones always on top */
@@ -967,18 +967,18 @@ void MainInterface::toggleUpdateSystrayMenu()
         for( hwnd = GetNextWindow( internalWinId(), GW_HWNDPREV );
                 hwnd && !IsWindowVisible( hwnd );
                 hwnd = GetNextWindow( hwnd, GW_HWNDPREV ) );
-        if( !hwnd || !GetWindowInfo( hwnd, &wi ) ||
+            if( !hwnd || !GetWindowInfo( hwnd, &wi ) ||
                 (wi.dwExStyle&WS_EX_TOPMOST) )
+            {
+                hide();
+            }
+            else
+            {
+                activateWindow();
+            }
 #else
-        if( isActiveWindow() )
+        hide();
 #endif
-        {
-            hide();
-        }
-        else
-        {
-            activateWindow();
-        }
     }
     QVLCMenu::updateSystrayMenu( this, p_intf );
 }
