@@ -2,7 +2,7 @@
  * generic_layout.hpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: generic_layout.hpp 15481 2006-04-30 18:07:40Z asmax $
+ * $Id: 7574f1fc087ca01f9e4fd5cd2bd05d8fbbcf4719 $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -36,6 +36,7 @@ class Anchor;
 class OSGraphics;
 class CtrlGeneric;
 class CtrlVideo;
+class VarBoolImpl;
 
 
 /// Control and its associated layer
@@ -52,7 +53,7 @@ struct LayeredControl
 
 
 /// Base class for layouts
-class GenericLayout: public SkinObject, public Box
+class GenericLayout: public SkinObject
 {
     public:
         GenericLayout( intf_thread_t *pIntf, int width, int height,
@@ -83,12 +84,20 @@ class GenericLayout: public SkinObject, public Box
         virtual OSGraphics *getImage() const { return m_pImage; }
 
         /// Get the position of the layout (relative to the screen)
+        /**
+         * Note: These values are different from the m_rect.getLeft() and
+         * m_rect.getTop(), which always return 0.
+         * The latter methods are there as a "root rect" for the panels and
+         * controls, since each control knows its parent rect, but returns
+         * coordinates relative to the root rect.
+         */
         virtual int getLeft() const { return m_pWindow->getLeft(); }
         virtual int getTop() const { return m_pWindow->getTop(); }
 
         /// Get the size of the layout
-        virtual int getWidth() const { return m_width; }
-        virtual int getHeight() const { return m_height; }
+        virtual int getWidth() const { return m_rect.getWidth(); }
+        virtual int getHeight() const { return m_rect.getHeight(); }
+        virtual const GenericRect &getRect() const { return m_rect; }
 
         /// Get the minimum and maximum size of the layout
         virtual int getMinWidth() const { return m_minWidth; }
@@ -99,8 +108,10 @@ class GenericLayout: public SkinObject, public Box
         /// Resize the layout
         virtual void resize( int width, int height );
 
-        /// Add a control in the layout at the given position, and
-        /// the optional given layer
+        /**
+         * Add a control in the layout at the given position, and
+         * the optional given layer
+         */
         virtual void addControl( CtrlGeneric *pControl,
                                  const Position &rPosition,
                                  int layer );
@@ -109,9 +120,11 @@ class GenericLayout: public SkinObject, public Box
         virtual const list<LayeredControl> &getControlList() const;
 
         /// Called by a control when its image has changed
-        /// The arguments indicate the size of the rectangle to refresh,
-        /// and the offset (from the control position) of this rectangle.
-        /// Use a negative width or height to refresh the layout completely
+        /**
+         * The arguments indicate the size of the rectangle to refresh,
+         * and the offset (from the control position) of this rectangle.
+         * Use a negative width or height to refresh the layout completely
+         */
         virtual void onControlUpdate( const CtrlGeneric &rCtrl,
                                       int width, int height,
                                       int xOffSet, int yOffSet );
@@ -128,23 +141,34 @@ class GenericLayout: public SkinObject, public Box
         /// Called when the layout is hidden
         virtual void onHide();
 
+        /// Give access to the "active layout" variable
+        // FIXME: we give read/write access
+        VarBoolImpl &getActiveVar() { return *m_pVarActive; }
+
     private:
         /// Parent window of the layout
         TopWindow *m_pWindow;
         /// Layout size
-        int m_width, m_height;
+        SkinsRect m_rect;
         int m_minWidth, m_maxWidth;
         int m_minHeight, m_maxHeight;
         /// Image of the layout
         OSGraphics *m_pImage;
         /// List of the controls in the layout
         list<LayeredControl> m_controlList;
-        //// Video control
+        /// Video control
         CtrlVideo *m_pVideoControl;
         /// List of the anchors in the layout
         list<Anchor*> m_anchorList;
         /// Flag to know if the layout is visible
         bool m_visible;
+        /// Variable for the "active state" of the layout
+        /**
+         * Note: the layout is not an observer on this variable, because it
+         * cannot be changed externally (i.e. without an explicit change of
+         * layout). This way, we avoid using a setActiveLayoutInner method.
+         */
+        mutable VarBoolImpl *m_pVarActive;
 };
 
 
