@@ -2,10 +2,10 @@
  * var_percent.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: var_percent.cpp 11786 2005-07-18 23:57:41Z asmax $
+ * $Id$
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include "var_percent.hpp"
@@ -43,7 +43,7 @@ void VarPercent::set( float percentage )
     if( m_value != percentage )
     {
         m_value = percentage;
-        notify();
+        notify( NULL );
     }
 }
 
