@@ -1,11 +1,11 @@
 /*****************************************************************************
  * volume.cpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: volume.cpp 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id$
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,21 +19,23 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#include <stdio.h>  // snprintf
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
-#include <vlc/aout.h>
+#include <vlc_common.h>
+#include <vlc_aout.h>
 #include "volume.hpp"
 
-
 Volume::Volume( intf_thread_t *pIntf ): VarPercent( pIntf )
 {
     // Initial value
     audio_volume_t val;
     aout_VolumeGet( getIntf(), &val );
-    VarPercent::set( val / AOUT_VOLUME_MAX );
+    VarPercent::set( val * 2.0 / AOUT_VOLUME_MAX );
 }
 
 
@@ -45,7 +47,7 @@ void Volume::set( float percentage )
     {
         VarPercent::set( percentage );
 
-        aout_VolumeSet( getIntf(), (int)(get() * AOUT_VOLUME_MAX) );
+        aout_VolumeSet( getIntf(), (int)(get() * AOUT_VOLUME_MAX / 2.0) );
     }
 }
 
