@@ -1,8 +1,8 @@
 /*****************************************************************************
  * video.cpp : WinCE gui plugin for VLC
  *****************************************************************************
- * Copyright (C) 2000-2004, 2003 VideoLAN
- * $Id: video.cpp 10153 2005-03-05 18:05:42Z gbazin $
+ * Copyright (C) 2000-2004, 2003 the VideoLAN team
+ * $Id: 6e307bae3ce777830b37fb58989f9ef7a15f0007 $
  *
  * Authors: Marodon Cedric <cedric_marodon@yahoo.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -19,15 +19,19 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <vlc/vlc.h>
-#include <vlc/vout.h>
-#include <vlc/intf.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_vout.h>
+#include <vlc_interface.h>
 
 #include "wince.h"
 
@@ -59,7 +63,7 @@ public:
                      unsigned int * );
     void ReleaseWindow( void * );
     int  ControlWindow( void *, int, va_list );
-        
+ 
     HWND p_child_window; // public because of menu
 
 private:
@@ -92,7 +96,7 @@ VideoWindow::VideoWindow( intf_thread_t *_p_intf, HWND _p_parent )
     p_parent = _p_parent;
     p_child_window = NULL;
 
-    vlc_mutex_init( p_intf, &lock );
+    vlc_mutex_init( &lock );
 
     p_vout = NULL;
 
@@ -188,7 +192,7 @@ void VideoWindow::ReleaseWindow( void *p_window )
 FUNCTION:
   WndProc
 
-PURPOSE: 
+PURPOSE:
   Processes messages sent to the main window.
 
 ***********************************************************************/
@@ -197,15 +201,13 @@ LRESULT VideoWindow::WndProc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
     switch( msg )
     {
     case WM_KILLFOCUS:
-        msg_Err( p_intf, "WM_KILLFOCUS1" );
         if( p_vout )
-            vout_Control( p_vout, VOUT_SET_FOCUS, (vlc_bool_t)VLC_FALSE );
+            vout_Control( p_vout, VOUT_SET_FOCUS, (bool)false );
         return TRUE;
 
     case WM_SETFOCUS:
-        msg_Err( p_intf, "WM_SETFOCUS1" );
         if( p_vout )
-            vout_Control( p_vout, VOUT_SET_FOCUS, (vlc_bool_t)VLC_TRUE );
+            vout_Control( p_vout, VOUT_SET_FOCUS, (bool)true );
         return TRUE;
 
     default:
@@ -229,9 +231,6 @@ int VideoWindow::ControlWindow( void *p_window, int i_query, va_list args )
 
     switch( i_query )
     {
-    case VOUT_SET_ZOOM:
-        break;
-
     case VOUT_SET_STAY_ON_TOP:
         break;
 
