@@ -2,7 +2,7 @@
  * audioscrobbler.c : audioscrobbler submission plugin
  *****************************************************************************
  * Copyright © 2006-2008 the VideoLAN team
- * $Id$
+ * $Id: 711fec7c961649406c9e47b5ad780f1f3659ac24 $
  *
  * Author: Rafaël Carré <funman at videolanorg>
  *
@@ -91,7 +91,7 @@ struct intf_sys_t
     int                     i_nowp_port;        /**< port to which submit   */
     char                    *psz_nowp_file;     /**< file to which submit   */
 #endif
-    bool              b_handshaked;       /**< are we authenticated ? */
+    bool                    b_handshaked;       /**< are we authenticated ? */
     char                    psz_auth_token[33]; /**< Authentication token */
 
     /* data about song currently playing */
@@ -100,12 +100,12 @@ struct intf_sys_t
     mtime_t                 time_pause;         /**< time when vlc paused   */
     mtime_t                 time_total_pauses;  /**< total time in pause    */
 
-    bool              b_submit;           /**< do we have to submit ? */
+    bool                    b_submit;           /**< do we have to submit ? */
 
-    bool              b_state_cb;         /**< if we registered the
+    bool                    b_state_cb;         /**< if we registered the
                                                  * "state" callback         */
 
-    bool              b_meta_read;        /**< if we read the song's
+    bool                    b_meta_read;        /**< if we read the song's
                                                  * metadata already         */
 };
 
@@ -325,6 +325,11 @@ static void Run( intf_thread_t *p_intf )
 
         msg_Dbg( p_intf, "Going to submit some data..." );
 
+        /* The session may be invalid if there is a trailing \n */
+        char *psz_ln = strrchr( p_sys->psz_auth_token, '\n' );
+        if( psz_ln )
+            *psz_ln = '\0';
+
         if( !asprintf( &psz_submit, "s=%s", p_sys->psz_auth_token ) )
         {   /* Out of memory */
             return;
@@ -337,14 +342,24 @@ static void Run( intf_thread_t *p_intf )
         {
             p_song = &p_sys->p_queue[i_song];
             if( !asprintf( &psz_submit_song,
-                    "&a%%5B%d%%5D=%s&t%%5B%d%%5D=%s"
-                    "&i%%5B%d%%5D=%ju&o%%5B%d%%5D=P&r%%5B%d%%5D="
-                    "&l%%5B%d%%5D=%d&b%%5B%d%%5D=%s"
-                    "&n%%5B%d%%5D=%s&m%%5B%d%%5D=%s",
-                    i_song, p_song->psz_a,           i_song, p_song->psz_t,
-                    i_song, (uintmax_t)p_song->date, i_song, i_song,
-                    i_song, p_song->i_l,             i_song, p_song->psz_b,
-                    i_song, p_song->psz_n,           i_song, p_song->psz_m
+                    "&a%%5B%d%%5D=%s"
+                    "&t%%5B%d%%5D=%s"
+                    "&i%%5B%d%%5D=%u"
+                    "&o%%5B%d%%5D=P"
+                    "&r%%5B%d%%5D="
+                    "&l%%5B%d%%5D=%d"
+                    "&b%%5B%d%%5D=%s"
+                    "&n%%5B%d%%5D=%s"
+                    "&m%%5B%d%%5D=%s",
+                    i_song, p_song->psz_a,
+                    i_song, p_song->psz_t,
+                    i_song, (unsigned)p_song->date, /* HACK: %ju (uintmax_t) unsupported on Windows */
+                    i_song,
+                    i_song,
+                    i_song, p_song->i_l,
+                    i_song, p_song->psz_b,
+                    i_song, p_song->psz_n,
+                    i_song, p_song->psz_m
             ) )
             {   /* Out of memory */
                 vlc_mutex_unlock( &p_sys->lock );
@@ -559,6 +574,19 @@ static void AddToQueue ( intf_thread_t *p_this )
                             p_sys->time_total_pauses;
     played_time /= 1000000; /* µs → s */
 
+    /*HACK: it seam that the preparsing sometime fail,
+            so use the playing time as the song length */
+    if( p_sys->p_current_song.i_l == 0 )
+        p_sys->p_current_song.i_l = played_time;
+
+    /* Don't send song shorter than 30s */
+    if( p_sys->p_current_song.i_l < 30 )
+    {
+        msg_Dbg( p_this, "Song too short (< 30s), not submitting" );
+        goto end;
+    }
+
+    /* Send if the user had listen more than 240s OR half the track length */
     if( ( played_time < 240 ) &&
         ( played_time < ( p_sys->p_current_song.i_l / 2 ) ) )
     {
@@ -566,17 +594,11 @@ static void AddToQueue ( intf_thread_t *p_this )
         goto end;
     }
 
-    if( p_sys->p_current_song.i_l < 30 )
-    {
-        msg_Dbg( p_this, "Song too short (< 30s), not submitting" );
-        goto end;
-    }
-
+    /* Check that all meta are present */
     if( !p_sys->p_current_song.psz_a || !*p_sys->p_current_song.psz_a ||
         !p_sys->p_current_song.psz_t || !*p_sys->p_current_song.psz_t )
     {
         msg_Dbg( p_this, "Missing artist or title, not submitting" );
-/*XXX*/        msg_Dbg( p_this, "%s %s", p_sys->p_current_song.psz_a, p_sys->p_current_song.psz_t );
         goto end;
     }
 
@@ -930,6 +952,8 @@ static int ReadMetaData( intf_thread_t *p_this )
         a = encode_URI_component( psz_meta ); \
         if( !a ) \
         { \
+            vlc_mutex_unlock( &p_sys->lock ); \
+            vlc_object_release( p_input ); \
             free( psz_meta ); \
             return VLC_ENOMEM; \
         } \
