@@ -2,7 +2,7 @@
  * misc.c
  *****************************************************************************
  * Copyright (C) 2007-2008 the VideoLAN team
- * $Id$
+ * $Id: 8188957406049dafb7183d1e50e26d9e5bc636ca $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *          Pierre d'Herbemont <pdherbemont # videolan.org>
@@ -171,7 +171,15 @@ static int vlclua_datadir_list( lua_State *L )
 static int vlclua_lock_and_wait( lua_State *L )
 {
     vlc_object_t *p_this = vlclua_get_this( L );
-    int b_quit = vlc_object_lock_and_wait( p_this );
+    int b_quit;
+
+    vlc_object_lock( p_this );
+    b_quit = vlc_object_alive( p_this );
+    if( b_quit )
+    {
+        vlc_object_wait( p_this );
+        b_quit = vlc_object_alive( p_this );
+    }
     lua_pushboolean( L, b_quit );
     return 1;
 }
@@ -189,6 +197,13 @@ static int vlclua_mdate( lua_State *L )
     return 1;
 }
 
+static int vlclua_mwait( lua_State *L )
+{
+    double f = luaL_checknumber( L, 1 );
+    mwait( (int64_t)f );
+    return 0;
+}
+
 static int vlclua_intf_should_die( lua_State *L )
 {
     intf_thread_t *p_intf = (intf_thread_t*)vlclua_get_this( L );
@@ -212,6 +227,7 @@ static const luaL_Reg vlclua_misc_reg[] = {
     { "datadir_list", vlclua_datadir_list },
 
     { "mdate", vlclua_mdate },
+    { "mwait", vlclua_mwait },
 
     { "lock_and_wait", vlclua_lock_and_wait },
     { "signal", vlclua_signal },
