@@ -2,7 +2,7 @@
  * meta.c: Get meta/artwork using lua scripts
  *****************************************************************************
  * Copyright (C) 2007-2008 the VideoLAN team
- * $Id$
+ * $Id: 80474c616b6b7e30aebb3eda57e984d8ec55fd7f $
  *
  * Authors: Antoine Cellerier <dionoea at videolan tod org>
  *          Pierre d'Herbemont <pdherbemont # videolan.org>
@@ -78,6 +78,9 @@ static lua_State * vlclua_meta_init( vlc_object_t *p_this, input_item_t * p_item
     luaopen_msg( L );
     luaopen_stream( L );
     luaopen_strings( L );
+    luaopen_variables( L );
+    luaopen_object( L );
+    luaopen_misc( L );
 
     lua_pushlightuserdata( L, p_this );
     lua_setfield( L, -2, "private" );
