@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2004-2005 the VideoLAN team
  * Copyright © 2007 Rémi Denis-Courmont
- * $Id: f7228b4f02bd54c02ea7c03a0a5d8435e341ef0a $
+ * $Id: f7a8261ae9f2ddbca27d39e34a94921ac8e6632d $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Rémi Denis-Courmont
@@ -287,6 +287,22 @@ static inline int min_int( int a, int b )
     return a > b ? b : a;
 }
 
+static bool IsWellKnownPayload (int type)
+{
+    switch (type)
+    {   /* Should be in sync with modules/demux/rtp.c */
+        case  0: /* PCMU/8000 */
+        case  8: /* PCMA/8000 */
+        case 10: /* L16/44100/2 */
+        case 11: /* L16/44100 */
+        case 14: /* MPA/90000 */
+        case 32: /* MPV/90000 */
+        case 33: /* MP2/90000 */
+            return true;
+   }
+   return false;
+}
+
 /*****************************************************************************
  * Open: initialize and create stuff
  *****************************************************************************/
@@ -394,19 +410,8 @@ static int OpenDemux( vlc_object_t *p_this )
     {
         p_sdp->psz_uri = NULL;
     }
-    switch (p_sdp->i_media_type)
-    {   /* Should be in sync with modules/demux/rtp.c */
-        case  0: /* PCMU/8000 */
-        case  8: /* PCMA/8000 */
-        case 10: /* L16/44100/2 */
-        case 11: /* L16/44100 */
-        case 14: /* MPA/90000 */
-        case 32: /* MPV/90000 */
-        case 33: /* MP2/90000 */
-            break;
-        default:
-            goto error;
-    }
+    if (!IsWellKnownPayload (p_sdp->i_media_type))
+        goto error;
     if( p_sdp->psz_uri == NULL ) goto error;
 
     p_demux->p_sys = (demux_sys_t *)malloc( sizeof(demux_sys_t) );
@@ -780,10 +785,7 @@ static int ParseSAP( services_discovery_t *p_sd, const uint8_t *buf,
         p_sdp->psz_uri = NULL;
 
     /* Multi-media or no-parse -> pass to LIVE.COM */
-    if( ( p_sdp->i_media_type != 14
-       && p_sdp->i_media_type != 32
-       && p_sdp->i_media_type != 33)
-     || p_sd->p_sys->b_parse == false )
+    if( !IsWellKnownPayload( p_sdp->i_media_type ) || !p_sd->p_sys->b_parse )
     {
         free( p_sdp->psz_uri );
         if (asprintf( &p_sdp->psz_uri, "sdp://%s", p_sdp->psz_sdp ) == -1)
@@ -1004,6 +1006,12 @@ static int ParseConnection( vlc_object_t *p_obj, sdp_t *p_sdp )
         return VLC_EGENERIC;
     }
 
+    if (strcmp (vlc_proto, "udp")
+     && (port & 1) /* odd media port? */
+     && !FindAttribute (p_sdp, 0, "rtcp-mux")
+     && !FindAttribute (p_sdp, 0, "rtcp"))
+        port++; /* RTP on next even port */
+
     if (flags & 1)
     {
         /* Connection-oriented media */
