@@ -1,8 +1,8 @@
 /*****************************************************************************
  * gather.c: gathering stream output module
  *****************************************************************************
- * Copyright (C) 2003-2004 VideoLAN
- * $Id: gather.c 10101 2005-03-02 16:47:31Z robux4 $
+ * Copyright (C) 2003-2004 the VideoLAN team
+ * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -18,17 +18,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>
 
-#include <vlc/vlc.h>
-#include <vlc/input.h>
-#include <vlc/sout.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_input.h>
+#include <vlc_sout.h>
 
 /*****************************************************************************
  * Module descriptor
@@ -37,11 +41,9 @@ static int      Open    ( vlc_object_t * );
 static void     Close   ( vlc_object_t * );
 
 vlc_module_begin();
-    set_description( _("Gathering stream output") );
+    set_description( N_("Gathering stream output") );
     set_capability( "sout stream", 50 );
     add_shortcut( "gather" );
-    set_category( CAT_SOUT );
-    set_subcategory( SUBCAT_SOUT_STREAM );
     set_callbacks( Open, Close );
 vlc_module_end();
 
@@ -50,12 +52,11 @@ vlc_module_end();
  *****************************************************************************/
 static sout_stream_id_t *Add ( sout_stream_t *, es_format_t * );
 static int               Del ( sout_stream_t *, sout_stream_id_t * );
-static int               Send( sout_stream_t *, sout_stream_id_t *,
-                               block_t* );
+static int               Send( sout_stream_t *, sout_stream_id_t *, block_t* );
 
 struct sout_stream_id_t
 {
-    vlc_bool_t    b_used;
+    bool    b_used;
 
     es_format_t fmt;
     void          *id;
@@ -78,19 +79,21 @@ static int Open( vlc_object_t *p_this )
     sout_stream_sys_t *p_sys;
 
     p_stream->p_sys = p_sys = malloc( sizeof( sout_stream_sys_t ) );
+    if( p_sys == NULL )
+        return VLC_EGENERIC;
+
     p_sys->p_out    = sout_StreamNew( p_stream->p_sout, p_stream->psz_next );
     if( p_sys->p_out == NULL )
     {
         free( p_sys );
         return VLC_EGENERIC;
     }
-    p_sys->i_id         = 0;
-    p_sys->id           = NULL;
-
     p_stream->pf_add    = Add;
     p_stream->pf_del    = Del;
     p_stream->pf_send   = Send;
 
+    TAB_INIT( p_sys->i_id, p_sys->id );
+
     return VLC_SUCCESS;
 }
 
@@ -105,10 +108,13 @@ static void Close( vlc_object_t * p_this )
 
     for( i = 0; i < p_sys->i_id; i++ )
     {
-        p_sys->p_out->pf_del( p_sys->p_out, p_sys->id[i]->id );
-        free( p_sys->id[i] );
+        sout_stream_id_t *id = p_sys->id[i];
+
+        sout_StreamIdDel( p_sys->p_out, id->id );
+        es_format_Clean( &id->fmt );
+        free( id );
     }
-    free( p_sys->id );
+    TAB_CLEAN( p_sys->i_id, p_sys->id );
 
     sout_StreamDelete( p_sys->p_out );
     free( p_sys );
@@ -123,35 +129,47 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
     sout_stream_id_t  *id;
     int i;
 
-    /* search a output compatible */
+    /* search a compatible output */
     for( i = 0; i < p_sys->i_id; i++ )
     {
         id = p_sys->id[i];
-        if( !id->b_used &&
-            id->fmt.i_cat == p_fmt->i_cat &&
-            id->fmt.i_codec == p_fmt->i_codec &&
-            ( ( id->fmt.i_cat == AUDIO_ES &&
-                id->fmt.audio.i_rate == p_fmt->audio.i_rate &&
-                id->fmt.audio.i_channels == p_fmt->audio.i_channels &&
-                id->fmt.audio.i_blockalign == p_fmt->audio.i_blockalign ) ||
-              ( id->fmt.i_cat == VIDEO_ES &&
-                id->fmt.video.i_width == p_fmt->video.i_width &&
-                id->fmt.video.i_height == p_fmt->video.i_height ) ) )
+        if( id->b_used )
+            continue;
+
+        if( id->fmt.i_cat != p_fmt->i_cat || id->fmt.i_codec != p_fmt->i_codec )
+            continue;
+
+        if( id->fmt.i_cat == AUDIO_ES )
         {
-            msg_Dbg( p_stream, "reusing already opened output" );
-            id->b_used = VLC_TRUE;
-            return id;
+            audio_format_t *p_a = &id->fmt.audio;
+            if( p_a->i_rate != p_fmt->audio.i_rate ||
+                p_a->i_channels != p_fmt->audio.i_channels ||
+                p_a->i_blockalign != p_fmt->audio.i_blockalign )
+                continue;
         }
+        else if( id->fmt.i_cat == VIDEO_ES )
+        {
+            video_format_t *p_v = &id->fmt.video;
+            if( p_v->i_width != p_fmt->video.i_width ||
+                p_v->i_height != p_fmt->video.i_height )
+                continue;
+        }
+
+        /* */
+        msg_Dbg( p_stream, "reusing already opened output" );
+        id->b_used = true;
+        return id;
     }
 
-    /* destroy all output of the same categorie */
+    /* destroy all outputs from the same category */
     for( i = 0; i < p_sys->i_id; i++ )
     {
         id = p_sys->id[i];
         if( !id->b_used && id->fmt.i_cat == p_fmt->i_cat )
         {
             TAB_REMOVE( p_sys->i_id, p_sys->id, id );
-            p_sys->p_out->pf_del( p_sys->p_out, id );
+            sout_StreamIdDel( p_sys->p_out, id->id );
+            es_format_Clean( &id->fmt );
             free( id );
 
             i = 0;
@@ -159,13 +177,13 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
         }
     }
 
-    id = malloc( sizeof( sout_stream_id_t ) );
     msg_Dbg( p_stream, "creating new output" );
-    memcpy( &id->fmt, p_fmt, sizeof( es_format_t ) );
-    id->fmt.i_extra = 0;
-    id->fmt.p_extra = NULL;
-    id->b_used           = VLC_TRUE;
-    id->id               = p_sys->p_out->pf_add( p_sys->p_out, p_fmt );
+    id = malloc( sizeof( sout_stream_id_t ) );
+    if( id == NULL )
+        return NULL;
+    es_format_Copy( &id->fmt, p_fmt );
+    id->b_used           = true;
+    id->id               = sout_StreamIdAdd( p_sys->p_out, &id->fmt );
     if( id->id == NULL )
     {
         free( id );
@@ -181,7 +199,8 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
  *****************************************************************************/
 static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
 {
-    id->b_used = VLC_FALSE;
+    VLC_UNUSED(p_stream);
+    id->b_used = false;
     return VLC_SUCCESS;
 }
 
@@ -193,5 +212,5 @@ static int Send( sout_stream_t *p_stream,
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
 
-    return p_sys->p_out->pf_send( p_sys->p_out, id->id, p_buffer );
+    return sout_StreamIdSend( p_sys->p_out, id->id, p_buffer );
 }
