@@ -2,7 +2,7 @@
  * croppadd.c: Crop/Padd image filter
  *****************************************************************************
  * Copyright (C) 2008 the VideoLAN team
- * $Id$
+ * $Id: 0139b71bda849d19219d261e68161414525da5c8 $
  *
  * Authors: Antoine Cellerier <dionoea @t videolan dot org>
  *
@@ -286,7 +286,7 @@ static picture_t *Filter( filter_t *p_filter, picture_t *p_pic )
 
             /* Padd on the right */
             vlc_memset( p_out, i_padd_color,
-                        ( i_outwidth - i_width ) * i_pixel_pitch );
+                        ( i_outwidth - i_xpadd - i_width ) * i_pixel_pitch );
 
             /* Got to begining of the next line */
             p_in = p_in_next;
