@@ -1,8 +1,8 @@
 /*****************************************************************************
  * intf.c : audio output API towards the interface modules
  *****************************************************************************
- * Copyright (C) 2002-2004 the VideoLAN team
- * $Id: intf.c 14953 2006-03-28 20:29:28Z zorglub $
+ * Copyright (C) 2002-2007 the VideoLAN team
+ * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -24,15 +24,20 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+
+#include <stdio.h>
 #include <stdlib.h>                            /* calloc(), malloc(), free() */
 #include <string.h>
 
-#include <vlc/vlc.h>
-
-#include "audio_output.h"
+#include <vlc_aout.h>
 #include "aout_internal.h"
 
-
 /*
  * Volume management
  *
@@ -61,21 +66,28 @@
  *****************************************************************************/
 int __aout_VolumeGet( vlc_object_t * p_object, audio_volume_t * pi_volume )
 {
-    int i_volume, i_result = 0;
+    int i_result = 0;
     aout_instance_t * p_aout = vlc_object_find( p_object, VLC_OBJECT_AOUT,
                                                 FIND_ANYWHERE );
 
-    i_volume = config_GetInt( p_object, "volume" );
-    if ( pi_volume != NULL ) *pi_volume = (audio_volume_t)i_volume;
+    if ( pi_volume == NULL ) return -1;
 
-    if ( p_aout == NULL ) return 0;
+    if ( p_aout == NULL )
+    {
+        *pi_volume = (audio_volume_t)config_GetInt( p_object, "volume" );
+        return 0;
+    }
 
-    vlc_mutex_lock( &p_aout->mixer_lock );
+    aout_lock_mixer( p_aout );
     if ( !p_aout->mixer.b_error )
     {
         i_result = p_aout->output.pf_volume_get( p_aout, pi_volume );
     }
-    vlc_mutex_unlock( &p_aout->mixer_lock );
+    else
+    {
+        *pi_volume = (audio_volume_t)config_GetInt( p_object, "volume" );
+    }
+    aout_unlock_mixer( p_aout );
 
     vlc_object_release( p_aout );
     return i_result;
@@ -92,17 +104,17 @@ int __aout_VolumeSet( vlc_object_t * p_object, audio_volume_t i_volume )
 
     config_PutInt( p_object, "volume", i_volume );
 
-    val.b_bool = VLC_TRUE;
-    var_Set( p_object->p_vlc, "volume-change", val );
+    val.b_bool = true;
+    var_Set( p_object->p_libvlc, "volume-change", val );
 
     if ( p_aout == NULL ) return 0;
 
-    vlc_mutex_lock( &p_aout->mixer_lock );
+    aout_lock_mixer( p_aout );
     if ( !p_aout->mixer.b_error )
     {
         i_result = p_aout->output.pf_volume_set( p_aout, i_volume );
     }
-    vlc_mutex_unlock( &p_aout->mixer_lock );
+    aout_unlock_mixer( p_aout );
 
     var_Set( p_aout, "intf-change", val );
     vlc_object_release( p_aout );
@@ -120,7 +132,7 @@ int __aout_VolumeInfos( vlc_object_t * p_object, audio_volume_t * pi_soft )
 
     if ( p_aout == NULL ) return 0;
 
-    vlc_mutex_lock( &p_aout->mixer_lock );
+    aout_lock_mixer( p_aout );
     if ( p_aout->mixer.b_error )
     {
         /* The output module is destroyed. */
@@ -130,7 +142,7 @@ int __aout_VolumeInfos( vlc_object_t * p_object, audio_volume_t * pi_soft )
     {
         i_result = p_aout->output.pf_volume_infos( p_aout, pi_soft );
     }
-    vlc_mutex_unlock( &p_aout->mixer_lock );
+    aout_unlock_mixer( p_aout );
 
     vlc_object_release( p_aout );
     return i_result;
@@ -145,11 +157,12 @@ int __aout_VolumeInfos( vlc_object_t * p_object, audio_volume_t * pi_soft )
 int __aout_VolumeUp( vlc_object_t * p_object, int i_nb_steps,
                    audio_volume_t * pi_volume )
 {
+    vlc_value_t val;
     aout_instance_t * p_aout = vlc_object_find( p_object, VLC_OBJECT_AOUT,
                                                 FIND_ANYWHERE );
     int i_result = 0, i_volume = 0, i_volume_step = 0;
 
-    i_volume_step = config_GetInt( p_object->p_vlc, "volume-step" );
+    i_volume_step = config_GetInt( p_object->p_libvlc, "volume-step" );
     i_volume = config_GetInt( p_object, "volume" );
     i_volume += i_volume_step * i_nb_steps;
     if ( i_volume > AOUT_VOLUME_MAX )
@@ -162,15 +175,18 @@ int __aout_VolumeUp( vlc_object_t * p_object, int i_nb_steps,
                     (audio_volume_t) i_volume );
     if ( pi_volume != NULL ) *pi_volume = (audio_volume_t) i_volume;
 
+    val.b_bool = true;
+    var_Set( p_object->p_libvlc, "volume-change", val );
+
     if ( p_aout == NULL ) return 0;
 
-    vlc_mutex_lock( &p_aout->mixer_lock );
+    aout_lock_mixer( p_aout );
     if ( !p_aout->mixer.b_error )
     {
         i_result = p_aout->output.pf_volume_set( p_aout,
                                                 (audio_volume_t) i_volume );
     }
-    vlc_mutex_unlock( &p_aout->mixer_lock );
+    aout_unlock_mixer( p_aout );
 
     vlc_object_release( p_aout );
     return i_result;
@@ -185,11 +201,12 @@ int __aout_VolumeUp( vlc_object_t * p_object, int i_nb_steps,
 int __aout_VolumeDown( vlc_object_t * p_object, int i_nb_steps,
                      audio_volume_t * pi_volume )
 {
+    vlc_value_t val;
     aout_instance_t * p_aout = vlc_object_find( p_object, VLC_OBJECT_AOUT,
                                                 FIND_ANYWHERE );
     int i_result = 0, i_volume = 0, i_volume_step = 0;
 
-    i_volume_step = config_GetInt( p_object->p_vlc, "volume-step" );
+    i_volume_step = config_GetInt( p_object->p_libvlc, "volume-step" );
     i_volume = config_GetInt( p_object, "volume" );
     i_volume -= i_volume_step * i_nb_steps;
     if ( i_volume < AOUT_VOLUME_MIN )
@@ -201,14 +218,17 @@ int __aout_VolumeDown( vlc_object_t * p_object, int i_nb_steps,
     var_SetInteger( p_object->p_libvlc, "saved-volume", (audio_volume_t) i_volume );
     if ( pi_volume != NULL ) *pi_volume = (audio_volume_t) i_volume;
 
+    val.b_bool = true;
+    var_Set( p_object->p_libvlc, "volume-change", val );
+
     if ( p_aout == NULL ) return 0;
 
-    vlc_mutex_lock( &p_aout->mixer_lock );
+    aout_lock_mixer( p_aout );
     if ( !p_aout->mixer.b_error )
     {
         i_result = p_aout->output.pf_volume_set( p_aout, (audio_volume_t) i_volume );
     }
-    vlc_mutex_unlock( &p_aout->mixer_lock );
+    aout_unlock_mixer( p_aout );
 
     vlc_object_release( p_aout );
     return i_result;
@@ -277,6 +297,7 @@ void aout_VolumeSoftInit( aout_instance_t * p_aout )
 /* Placeholder for pf_volume_infos(). */
 int aout_VolumeSoftInfos( aout_instance_t * p_aout, audio_volume_t * pi_soft )
 {
+    (void)p_aout;
     *pi_soft = 0;
     return 0;
 }
@@ -313,18 +334,21 @@ void aout_VolumeNoneInit( aout_instance_t * p_aout )
 /* Placeholder for pf_volume_infos(). */
 int aout_VolumeNoneInfos( aout_instance_t * p_aout, audio_volume_t * pi_soft )
 {
+    (void)p_aout; (void)pi_soft;
     return -1;
 }
 
 /* Placeholder for pf_volume_get(). */
 int aout_VolumeNoneGet( aout_instance_t * p_aout, audio_volume_t * pi_volume )
 {
+    (void)p_aout; (void)pi_volume;
     return -1;
 }
 
 /* Placeholder for pf_volume_set(). */
 int aout_VolumeNoneSet( aout_instance_t * p_aout, audio_volume_t i_volume )
 {
+    (void)p_aout; (void)i_volume;
     return -1;
 }
 
@@ -340,24 +364,26 @@ int aout_VolumeNoneSet( aout_instance_t * p_aout, audio_volume_t i_volume )
  * This function is used whenever the parameters of the output plug-in are
  * changed (eg. selecting S/PDIF or PCM).
  *****************************************************************************/
-int aout_Restart( aout_instance_t * p_aout )
+static int aout_Restart( aout_instance_t * p_aout )
 {
     int i;
-    vlc_bool_t b_error = 0;
+    bool b_error = 0;
 
-    vlc_mutex_lock( &p_aout->mixer_lock );
+    aout_lock_mixer( p_aout );
 
     if ( p_aout->i_nb_inputs == 0 )
     {
-        vlc_mutex_unlock( &p_aout->mixer_lock );
+        aout_unlock_mixer( p_aout );
         msg_Err( p_aout, "no decoder thread" );
         return -1;
     }
 
     /* Lock all inputs. */
+    aout_lock_input_fifos( p_aout );
+
     for ( i = 0; i < p_aout->i_nb_inputs; i++ )
     {
-        vlc_mutex_lock( &p_aout->pp_inputs[i]->lock );
+        aout_lock_input( p_aout, p_aout->pp_inputs[i] );
         aout_InputDelete( p_aout, p_aout->pp_inputs[i] );
     }
 
@@ -373,7 +399,8 @@ int aout_Restart( aout_instance_t * p_aout )
         {
             vlc_mutex_unlock( &p_aout->pp_inputs[i]->lock );
         }
-        vlc_mutex_unlock( &p_aout->mixer_lock );
+        aout_unlock_input_fifos( p_aout );
+        aout_unlock_mixer( p_aout );
         return -1;
     }
 
@@ -384,7 +411,8 @@ int aout_Restart( aout_instance_t * p_aout )
         {
             vlc_mutex_unlock( &p_aout->pp_inputs[i]->lock );
         }
-        vlc_mutex_unlock( &p_aout->mixer_lock );
+        aout_unlock_input_fifos( p_aout );
+        aout_unlock_mixer( p_aout );
         return -1;
     }
 
@@ -392,13 +420,13 @@ int aout_Restart( aout_instance_t * p_aout )
     for ( i = 0; i < p_aout->i_nb_inputs; i++ )
     {
         aout_input_t * p_input = p_aout->pp_inputs[i];
-
         b_error |= aout_InputNew( p_aout, p_input );
         p_input->b_changed = 1;
-        vlc_mutex_unlock( &p_input->lock );
+        aout_unlock_input( p_aout, p_input );
     }
 
-    vlc_mutex_unlock( &p_aout->mixer_lock );
+    aout_unlock_input_fifos( p_aout );
+    aout_unlock_mixer( p_aout );
 
     return b_error;
 }
@@ -411,11 +439,12 @@ int aout_Restart( aout_instance_t * p_aout )
  * rebuilding the audio-device and audio-channels variables.
  *****************************************************************************/
 int aout_FindAndRestart( vlc_object_t * p_this, const char *psz_name,
-                         vlc_value_t oldval, vlc_value_t val, void *p_data )
+                         vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
     aout_instance_t * p_aout = vlc_object_find( p_this, VLC_OBJECT_AOUT,
                                                 FIND_ANYWHERE );
 
+    (void)psz_name; (void)oldval; (void)newval; (void)p_data;
     if ( p_aout == NULL ) return VLC_SUCCESS;
 
     if ( var_Type( p_aout, "audio-device" ) != 0 )
@@ -437,10 +466,11 @@ int aout_FindAndRestart( vlc_object_t * p_this, const char *psz_name,
  * aout_ChannelsRestart : change the audio device or channels and restart
  *****************************************************************************/
 int aout_ChannelsRestart( vlc_object_t * p_this, const char * psz_variable,
-                          vlc_value_t old_value, vlc_value_t new_value,
-                          void * unused )
+                          vlc_value_t oldval, vlc_value_t newval,
+                          void *p_data )
 {
     aout_instance_t * p_aout = (aout_instance_t *)p_this;
+    (void)oldval; (void)newval; (void)p_data;
 
     if ( !strcmp( psz_variable, "audio-device" ) )
     {
@@ -454,3 +484,36 @@ int aout_ChannelsRestart( vlc_object_t * p_this, const char * psz_variable,
     aout_Restart( p_aout );
     return 0;
 }
+
+/** Enable or disable an audio filter
+ * \param p_this a vlc object
+ * \param psz_name name of the filter
+ * \param b_add are we adding or removing the filter ?
+ */
+void aout_EnableFilter( vlc_object_t *p_this, const char *psz_name,
+                        bool b_add )
+{
+    aout_instance_t *p_aout = vlc_object_find( p_this, VLC_OBJECT_AOUT,
+                                               FIND_ANYWHERE );
+
+    if( AoutChangeFilterString( p_this, p_aout, "audio-filter", psz_name, b_add ) )
+    {
+        if( p_aout )
+            AoutInputsMarkToRestart( p_aout );
+    }
+
+    if( p_aout )
+        vlc_object_release( p_aout );
+}
+
+/**
+ * Change audio visualization
+ * -1 goes backwards, +1 goes forward
+ */
+char *aout_VisualChange( vlc_object_t *p_this, int i_skip )
+{
+    (void)p_this; (void)i_skip;
+    msg_Err( p_this, "FIXME: %s (%s %d) isn't implemented.", __func__,
+             __FILE__, __LINE__ );
+    return strdup("foobar");
+}
