@@ -1,8 +1,8 @@
 /*****************************************************************************
  * access.c
  *****************************************************************************
- * Copyright (C) 1999-2004 VideoLAN
- * $Id: demux.c 7546 2004-04-29 13:53:29Z gbazin $
+ * Copyright (C) 1999-2004 the VideoLAN team
+ * $Id: 48bd44eb2a94e1f749bc00999630723985671525 $
  *
  * Author: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -18,47 +18,45 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#include <stdlib.h>
-#include <vlc/vlc.h>
-#include <vlc/input.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
 
 #include "input_internal.h"
 
 /*****************************************************************************
- * access2_InternalNew:
+ * access_InternalNew:
  *****************************************************************************/
-static access_t *access2_InternalNew( vlc_object_t *p_obj,
-                                      char *psz_access, char *psz_demux, char *psz_path,
-                                      access_t *p_source,
-                                      vlc_bool_t b_quick )
+static access_t *access_InternalNew( vlc_object_t *p_obj, const char *psz_access,
+                                      const char *psz_demux, const char *psz_path,
+                                      access_t *p_source )
 {
-    access_t *p_access = vlc_object_create( p_obj, VLC_OBJECT_ACCESS );
+    static const char typename[] = "access";
+    access_t *p_access = vlc_custom_create( p_obj, sizeof (*p_access),
+                                            VLC_OBJECT_GENERIC, typename );
 
     if( p_access == NULL )
-    {
-        msg_Err( p_obj, "vlc_object_create( p_obj, VLC_OBJECT_ACCESS ) failed" );
         return NULL;
-    }
 
     /* Parse URL */
     p_access->p_source = p_source;
     if( p_source )
     {
         msg_Dbg( p_obj, "creating access filter '%s'", psz_access );
-        p_access->psz_access = strdup( p_source->psz_access );
-        p_access->psz_path   = strdup( p_source->psz_path );
     }
-    else if( !b_quick )
+    else
     {
         msg_Dbg( p_obj, "creating access '%s' path='%s'",
                  psz_access, psz_path );
-        p_access->psz_access = b_quick ? strdup( "file" ) : strdup( psz_access );
         p_access->psz_path   = strdup( psz_path );
     }
-    p_access->psz_demux  = strdup( "" );
+    p_access->psz_access = strdup( psz_access );
+    p_access->psz_demux  = strdup( psz_demux );
 
     p_access->pf_read    = NULL;
     p_access->pf_block   = NULL;
@@ -68,7 +66,8 @@ static access_t *access2_InternalNew( vlc_object_t *p_obj,
     p_access->info.i_update = 0;
     p_access->info.i_size   = 0;
     p_access->info.i_pos    = 0;
-    p_access->info.b_eof    = VLC_FALSE;
+    p_access->info.b_eof    = false;
+    p_access->info.b_prebuffered = false;
     p_access->info.i_title  = 0;
     p_access->info.i_seekpoint = 0;
 
@@ -76,25 +75,18 @@ static access_t *access2_InternalNew( vlc_object_t *p_obj,
     /* Before module_Need (for var_Create...) */
     vlc_object_attach( p_access, p_obj );
 
-    if( p_source )
-    {
-        p_access->p_module =
-            module_Need( p_access, "access_filter", psz_access, VLC_FALSE );
-    }
-    else
-    {
-        p_access->p_module =
-            module_Need( p_access, "access2",p_access->psz_access,
-                         b_quick ? VLC_TRUE : VLC_FALSE );
-    }
+    p_access->p_module =
+         module_Need( p_access, p_source ? "access_filter" : "access",
+                      psz_access, true );
 
     if( p_access->p_module == NULL )
     {
+        msg_StackAdd( "could not create access" );
         vlc_object_detach( p_access );
         free( p_access->psz_access );
         free( p_access->psz_path );
         free( p_access->psz_demux );
-        vlc_object_destroy( p_access );
+        vlc_object_release( p_access );
         return NULL;
     }
 
@@ -102,29 +94,29 @@ static access_t *access2_InternalNew( vlc_object_t *p_obj,
 }
 
 /*****************************************************************************
- * access2_New:
+ * access_New:
  *****************************************************************************/
-access_t *__access2_New( vlc_object_t *p_obj,
-                         char *psz_access, char *psz_demux, char *psz_path,
-                         vlc_bool_t b_quick )
+access_t *__access_New( vlc_object_t *p_obj, const char *psz_access,
+                         const char *psz_demux, const char *psz_path )
 {
-    return access2_InternalNew( p_obj, psz_access, psz_demux, psz_path, NULL, b_quick );
+    return access_InternalNew( p_obj, psz_access, psz_demux,
+                                psz_path, NULL );
 }
 
 /*****************************************************************************
- * access2_FilterNew:
+ * access_FilterNew:
  *****************************************************************************/
-access_t *access2_FilterNew( access_t *p_source, char *psz_access_filter )
+access_t *access_FilterNew( access_t *p_source, const char *psz_access_filter )
 {
-    return access2_InternalNew( VLC_OBJECT(p_source),
-                                psz_access_filter, NULL, NULL,
-                                p_source, VLC_FALSE );
+    return access_InternalNew( VLC_OBJECT(p_source), psz_access_filter,
+                                p_source->psz_demux, p_source->psz_path,
+                                p_source );
 }
 
 /*****************************************************************************
- * access2_Delete:
+ * access_Delete:
  *****************************************************************************/
-void access2_Delete( access_t *p_access )
+void access_Delete( access_t *p_access )
 {
     module_Unneed( p_access, p_access->p_module );
     vlc_object_detach( p_access );
@@ -135,9 +127,9 @@ void access2_Delete( access_t *p_access )
 
     if( p_access->p_source )
     {
-        access2_Delete( p_access->p_source );
+        access_Delete( p_access->p_source );
     }
 
-    vlc_object_destroy( p_access );
+    vlc_object_release( p_access );
 }
 
