@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001, 2002 the VideoLAN team
  * Copyright © 2006-2007 Rémi Denis-Courmont
- * $Id$
+ * $Id: 4738306db10acfb8856a042cc162d54b234a018c $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -204,16 +204,6 @@ struct vlc_object_internals_t
 
 #define vlc_internals( obj ) (((vlc_object_internals_t*)(VLC_OBJECT(obj)))-1)
 
-/* Signal an object without checking for locking consistency. This is wrong. */
-#ifdef __GNUC__
-__attribute__((deprecated))
-#endif
-static inline void
-vlc_object_signal_maybe (vlc_object_t *p_this)
-{
-    vlc_cond_signal (&(vlc_internals(p_this)->wait));
-}
-
 /**
  * Private LibVLC instance data.
  */
