@@ -1,8 +1,8 @@
 /*****************************************************************************
  * search.c : Search functions
  *****************************************************************************
- * Copyright (C) 1999-2004 the VideoLAN team
- * $Id$
+ * Copyright (C) 1999-2009 the VideoLAN team
+ * $Id: 720f4817b3f8d92934cae91b2e98690361990e88 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -126,9 +126,30 @@ playlist_item_t * playlist_ItemGetByInputId( playlist_t *p_playlist,
  * Live search handling
  ***************************************************************************/
 
-static bool playlist_LiveSearchUpdateInternal( playlist_t *p_playlist,
-                                                     playlist_item_t *p_root,
-                                                     const char *psz_string )
+/**
+ * Enable all items in the playlist
+ * @param p_root: the current root item
+ */
+static void playlist_LiveSearchClean( playlist_item_t *p_root )
+{
+    for( int i = 0; i < p_root->i_children; i++ )
+    {
+        playlist_item_t *p_item = p_root->pp_children[i];
+        if( p_item->i_children >= 0 )
+            playlist_LiveSearchClean( p_item );
+        p_item->i_flags &= ~PLAYLIST_DBL_FLAG;
+    }
+}
+
+
+/**
+ * Enable/Disable items in the playlist according to the search argument
+ * @param p_root: the current root item
+ * @param psz_string: the string to search
+ * @return true if an item match
+ */
+static bool playlist_LiveSearchUpdateInternal( playlist_item_t *p_root,
+                                               const char *psz_string )
 {
    int i;
    bool b_match = false;
@@ -137,7 +158,7 @@ static bool playlist_LiveSearchUpdateInternal( playlist_t *p_playlist,
         playlist_item_t *p_item = p_root->pp_children[i];
         if( p_item->i_children > -1 )
         {
-            if( playlist_LiveSearchUpdateInternal( p_playlist, p_item, psz_string ) ||
+            if( playlist_LiveSearchUpdateInternal( p_item, psz_string ) ||
                 strcasestr( p_item->p_input->psz_name, psz_string ) )
             {
                 p_item->i_flags &= ~PLAYLIST_DBL_FLAG;
@@ -150,7 +171,7 @@ static bool playlist_LiveSearchUpdateInternal( playlist_t *p_playlist,
         }
         else
         {
-            if( strcasestr( p_item->p_input->psz_name, psz_string ) || /* Soon to be replaced by vlc_meta_Title */
+            if( input_item_MetaMatch( p_item->p_input, vlc_meta_Title, psz_string ) ||
                 input_item_MetaMatch( p_item->p_input, vlc_meta_Album, psz_string ) ||
                 input_item_MetaMatch( p_item->p_input, vlc_meta_Artist, psz_string ) )
             {
@@ -171,7 +192,11 @@ int playlist_LiveSearchUpdate( playlist_t *p_playlist, playlist_item_t *p_root,
 {
     PL_ASSERT_LOCKED;
     p_playlist->b_reset_currently_playing = true;
-    playlist_LiveSearchUpdateInternal( p_playlist, p_root, psz_string );
+    if( *psz_string )
+        playlist_LiveSearchUpdateInternal( p_root, psz_string );
+    else
+        playlist_LiveSearchClean( p_root );
     vlc_object_signal_unlocked( p_playlist );
     return VLC_SUCCESS;
 }
+
