@@ -6,7 +6,7 @@
  * thread, and destroy a previously oppened video output thread.
  *****************************************************************************
  * Copyright (C) 2000-2007 the VideoLAN team
- * $Id: f54ec25fa1e35cadff4904450529563cdd6eed99 $
+ * $Id: b41aad2b61af8b6c0f8ca2b9d68541f915492148 $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -449,9 +449,7 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent, video_format_t *p_fmt )
     if( p_vout->p_module == NULL )
     {
         msg_Err( p_vout, "no suitable vout module" );
-        // FIXME it's ugly but that's exactly the function that need to be called.
-        EndThread( p_vout );
-        vlc_object_detach( p_vout );
+        vlc_object_set_destructor( p_vout, vout_Destructor );
         vlc_object_release( p_vout );
         return NULL;
     }
@@ -504,6 +502,8 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent, video_format_t *p_fmt )
                            VLC_THREAD_PRIORITY_OUTPUT, true ) )
     {
         module_Unneed( p_vout, p_vout->p_module );
+        p_vout->p_module = NULL;
+        vlc_object_set_destructor( p_vout, vout_Destructor );
         vlc_object_release( p_vout );
         return NULL;
     }
@@ -546,6 +546,9 @@ static void vout_Destructor( vlc_object_t * p_this )
     /* Make sure the vout was stopped first */
     assert( !p_vout->p_module );
 
+    /* */
+    spu_Destroy( p_vout->p_spu );
+
     /* Destroy the locks */
     vlc_mutex_destroy( &p_vout->picture_lock );
     vlc_mutex_destroy( &p_vout->change_lock );
@@ -972,7 +975,7 @@ static void* RunThread( vlc_object_t *p_this )
         {
             p_input = vlc_object_find( p_vout, VLC_OBJECT_INPUT, FIND_PARENT );
             p_subpic = spu_SortSubpictures( p_vout->p_spu, display_date,
-                                            p_input ? var_GetBool( p_input, "state" ) == PAUSE_S : false );
+                                            p_input ? var_GetInteger( p_input, "state" ) == PAUSE_S : false );
             if( p_input )
                 vlc_object_release( p_input );
         }
@@ -1248,7 +1251,6 @@ static void EndThread( vout_thread_t *p_vout )
 
     /* Destroy subpicture unit */
     spu_Attach( p_vout->p_spu, VLC_OBJECT(p_vout), false );
-    spu_Destroy( p_vout->p_spu );
 
     /* Destroy the video filters2 */
     filter_chain_Delete( p_vout->p_vf2_chain );
